getMissingData <- function(data_trial, rightSentenceBorder, PIX_PER_CHAR)
{
# define missing data, ignore missings after rightSentenceBorder
#-------------------------------------------------------------------
#  (matlab) Version 1.0, 02 JUNE 2005
#  by antje nuthmann
#
# (blind) port to R by Jochen Laubrock
#-------------------------------------------------------------------
#
#  INPUT:
#
#  data_trial       carries the raw data of a given trial
#  rightSentenceBorder right sentence border 
#  PIX_PER_CHAR     pixels per character (how many pixel make up a character)
#
#  OUTPUT:
#
#  missing          is nonempty if missing values within this trial
#  data_trial       adjusted, if necessary (default: no)
#---------------------------------------------------------------------
#
# idea: refinement of missing data treatment since blinks during the saccade to the lower right corner are of no harm
# (cf., pupil data analysis) for doing so, the right sentence border as well as the
# timepoint the eyes crossed that border have to be known

# NB: isn't this overly sophisticated (jl)?
missing <- NULL
rightCrit <- rightSentenceBorder + PIX_PER_CHAR
if (ncol(data_trial) == 7) {
	misl <- which(data_trial[,2] > rightCrit)   # x pos. of left eye
	misr <- which(data_trial[,5] > rightCrit)    # x pos. of right eye
	if (length(misl)>0 & length(misr)>0) {    
		# if both eyes have crossed the right sentence border, use the higher value (misl(1) or misr(1))
		# this is the row/moment in time where also the 'slower' eye crossed the right sentence border
	    mis <- max(c(misl[1], misr[1]))  
	    # remove all data after missings / blinks after sentence
	    idx_spec <- data_trial[mis:nrow(data_trial), 2] < 0 | data_trial[mis:nrow(data_trial), 5] < 0
	    if (sum(idx_spec)>0) {
	        data_trial <- data_trial[1:mis,]
	    }
	}
	missing <- which(data_trial[,2] < 0 | data_trial[,5] < 0) 
} else if (ncol(data_trial) == 4) {
	misx <- which(data_trial[,2] > rightCrit)   # x pos. of eye
	if (length(misx)>0) {    
	    mis <- misx[1]
	    idx_spec <- data_trial[mis:nrow(data_trial), 2] < 0
	    if (sum(idx_spec)>0) {
	        data_trial <- data_trial[1:mis,]
	    }
	}
	missing <- which(data_trial[,2] < 0)
} else { stop('getMissingData: wrong format of data_trial (need 4 or 7 columns)') }
res = list(missing = missing, data_trial = data_trial)
}