`getMissingDataAAA` <- function(data_trial, leftSentenceBorder, rightSentenceBorder, PIX_PER_CHAR)
{
# define missing data, ignore missings after rightSentenceBorder
#  INPUT:
#
#  data_trial       carries the raw data of a given trial; needs to provide x and y gaze coordinates in columns 2 and 3
 # leftSentenceBorder  left sentence border (xpos in pix)
#  rightSentenceBorder right sentence border (xpos in pix)
#  PIX_PER_CHAR     pixels per character (how many pixel make up a character)
#
#  OUTPUT:
#  missing          is nonempty if missing values within this trial
#  prepost			numerical index into trial-inital/-final samples marked as invalid because their location was before first and after last word
#  data_trial       adjusted, if necessary (default: no)
missing <- NULL
nblinks <- 0
prepost <- NULL
rightCrit <- rightSentenceBorder + PIX_PER_CHAR
mis0 <- NULL
mis1 <- NULL
if (ncol(data_trial) == 4) {
	# make sure that first word was fixated (heuristically use 5-char criterion)
	# leftOf5charsIn <- (data_trial[,2] < leftSentenceBorder - 5*PIX_PER_CHAR) & (data_trial[,2]>0) & !is.na(data_trial[, 2])
	# if (any(leftOf5charsIn) & which(leftOf5charsIn)<=100) {
	# 	ix0 <- min(which(leftOf5charsIn)) 
	# 	mis0 <- 1:ix0
	# 	data_trial[mis0, 2:ncol(data_trial)] <- NA
	# } else {
	# 	mis0 <- NULL
	# }
	

	# stop after gaze moved far (3 chars) out of last word, or large y saccade has started
	rightOf3charsOut <- (data_trial[,2] > rightSentenceBorder + 3*PIX_PER_CHAR) & !is.na(data_trial[, 2]) 
	severalLinesBelowAfterSentence <- (data_trial[,3] > median(data_trial[data_trial[,3]>0,3], na.rm=TRUE)  + 5*PIX_PER_CHAR) & data_trial[,2] > rightSentenceBorder & !is.na(data_trial[, 2]) 
	afterSentenceIx <- rightOf3charsOut | severalLinesBelowAfterSentence
	if (any(afterSentenceIx, na.rm=TRUE)) {
		ix1 <- min(which(afterSentenceIx)) 
		mis1 <- ix1:nrow(data_trial)
		data_trial[mis1, 2:ncol(data_trial)] <- NA
	} else {
		mis1 <- NULL
	}

	missing <- which(data_trial[,2] < 0 | data_trial[,3] < 0)
	DTCRIT <- 20  # minimum time in ms between two blinks to count them as separate
	if (length(missing)>0) {
		if (length(missing==1)) {
			nblinks <- 1
		} else {
			nblinks <- sum(parseq(diff(missing))[,3]>DTCRIT*SAMPLING/1000)
		}
	}
	prepost <- c(mis0, mis1)
} else { 
	stop('getMissingData: wrong format of data_trial (currently need 4 columns)') 
}
res = list(missing = missing, nblinks=nblinks, prepost=prepost, data_trial = data_trial)
}
