`preprocess` <- function (edfdir = '.', edf2ascpath='~/bin/'){
# 
#¬¥conversion of a bash script for preprocessing of EDF-files produced by eyelink (2 or 1000)
# (bash script, jochen laubrock, 08/2006)
# port to R, jochen laubrock, 2009
# Input
# 	edfdir: directory containing EDF-files
#	edf2ascpath: OS- and installation-dependent path to edf2asc binary
#

olddirectory <- setwd(edfdir)
#rename to lower-case (for aesthetical reasons)
edffilelist <- dir()
EDFfilelist <- edffilelist[grep('.EDF$', edffilelist, useBytes=TRUE)]
for (f in EDFfilelist) {
	file.rename(f, tolower(f))
}
edffilelist <- dir()
edffilelist <- edffilelist[grep('.[eE][dD][fF]$', edffilelist, useBytes=TRUE)]

#convert to ascii
if (!file.exists('dat')) {dir.create('dat')}
if (!file.exists('msg')) {dir.create('msg')}
for (f in edffilelist) {
	system(paste(edf2ascpath, 'edf2asc -p dat -s -miss -1.0 ', f, sep=''))
	system(paste(edf2ascpath, 'edf2asc -p msg -e -miss -1.0 ', f, sep=''))
}

#rename message files to msg and dat files to dat
dirs <- c('msg', 'dat')
for (suffix in dirs) {
	tmpdir <- setwd(suffix)
	theList <- dir()	
	for (f in theList) {
		newname <- sub('asc$', suffix, f)
		file.rename(f, tolower(newname))
	}
	setwd(tmpdir)
}

#move edf files to processed directoryectory
if (!file.exists('processed')) {dir.create('processed')}
for (f in edffilelist) {
	file.rename(f, paste('processed',f, sep='/'))
}


#  remove irrelevant lines (calibration, validation, fixation, saccade, blink events) from msg files
#  TODO: in another set of analyses,  evaluate SR parser messages and compare their saccade detection to ours
msgfilelist <- sub('.msg','',dir('msg', pattern='msg$'))
for (f in msgfilelist) {
	infilename <- paste('msg/', f, '.msg', sep="")
	outfilename <- paste('msg/', f, '.msg2', sep="")
	print(infilename)
	msg <- readLines(infilename)
	msg <- msg[-grep('(!CAL)|(VALIDATE)|(DRIFTCORRECT)|(THRESHOLDS)|(RECCFG)|(GAZE_COORDS)|(MODE)|(BLINK)|(PUPIL)|(ELCL*)])', msg, useBytes=TRUE)]
	msg <- msg[grep('(MSG)|(START)|(END)', msg, useBytes=TRUE)]
	writeLines(msg, con = outfilename)
}

# remove trial markers, empty rows, and empty columns
# TODO: we currently read and write twice--first, to have a text representation and second a numerical representation. 
# TODO: save one of the steps, and do every processing step on the text representation
print('Removing trialmarkers. Please be patient!')
datlist <- sub('.dat','',dir('dat'))
for (f in datlist){
	#remove stupid trialmarkes (# ~/bin/remove_trialmarkers.sh $i, where remove_trialmarkers is only 2 lines of shell code)
	filename <- paste('dat/', f, '.dat', sep="")
	print(filename)
	dat <- readLines(filename)
	invalidIx <- grep('^[\\..[[[:space:]]]*', dat, useBytes=TRUE) # find dot or space at beginning of line
	if (!is.null(dim(invalidIx))) {
		dat <- dat[-invalidIx]
	}
	# TODO: remove empty rows (indicated by dots)

	# remove "C" lines
	invalidIx <- grep('C',  dat, useBytes=TRUE)
	if (!is.null(dim(invalidIx))) {
		dat <- dat[-invalidIx]
	}
	# remove extra column
	dat <- gsub('\t\\.+', '', dat, perl=TRUE)

	writeLines(dat, con = filename)

	# remove extra column (alternative)
	# dat <- read.table(filename, nrow=length(dat))
	# ix <- NULL
	# for (i in 1:ncol(dat)) {
	# 	if (is.factor(dat[,i])) { ix <- rbind(ix, i) }
	# }
	# dat <- dat[,-ix]
	# if (!is.null(ix)) {
	# 	write.table(dat, file=filename, row.names=FALSE, col.names=FALSE)
	# }
}

setwd(olddirectory)
}