removeShortFixations <- function(a, MIN_DUR=14) #
{
# # remove extremely short fixations, likely to be 'overshoot-correction' artifacts (resulting from glissades)
# inputs: 
#	a: fixation sequence
#		labeled data frame, containing as a minimum the variables "dur", "sacdur", "x_component", "y_component", "xampl", "yampl", "vel"
# 	MIN_DUR: 10	# minimum duration of a fixation to keep
# output:
#	a: fixation sequence with short fixations removed
# 		an extra column "isModified" is added that has a value of TRUE for modified fixations, and FALSE otherwise
#
#
# TODO: allow user to choose to modify fixations instead of saccades so that
# glissade and short fixation duration are aded to the following fixation
#
# jochen laubrock, 08/02/08

relvars <- c("fdur", "sacdur", "x_component", "y_component", "xampl", "yampl", "peakvel")
b <- a[,relvars]
vshrtfix <- vector(mode = "logical", length = nrow(a))
vshrtfix <- b$dur < MIN_DUR
if (sum(vshrtfix)>0) {
	vshrtfixIx <- which(vshrtfix)
	vshrtfixSeq <- parseq(vshrtfixIx)
	b$isModified <- vshrtfix
	# we might need to consider this special case. right now we only print a diagnostic message
	if (sum(vshrtfixSeq[,3]>1)) {
		print("removeShortFixations: at least one sequence of two or more extremely short fixations exists")
	}

	# inspect a few cases	# n=57; a[(badfixIx[n]-2):(badfixIx[n]+2),]

	# now remove short fixations by adjusting neighboring fixations
	# case distinction
	# (1) first fixation in sentence: shift firstLastInSentence flag one up, then remove fixation (no adjustment of sacc pars needed)
	isFirstInSentence <- vshrtfixIx %in% which(a$firstLastInSentence==1)
	b[vshrtfixIx[isFirstInSentence]+1,"firstLastInSentence"] <- 1 # set following fix to be the first in sentence

	# (2) brief pseudo-fixation caused by corrective (?) saccade: add saccade duration to statistics of preceding sacc, then remove
	# note: there are some strange cases like id 58, sn 52 (short fix followed by long sacc) which require the argmax treatment, see below
	ix = vshrtfixIx[!isFirstInSentence]
	idxs = matrix(nrow=length(ix), ncol=3)
	cnt=1
	for (i in ix) {
		idxs[cnt,1] <- (which.max(abs(b[(i-1):i, "peakvel"])))
		idxs[cnt,2] <- (which.max(abs(b[(i-1):i, "xampl"])))
		idxs[cnt,3] <- (which.max(abs(b[(i-1):i, "yampl"])))
		cnt=cnt+1
	}
	
	b[ix-1, "fdur"] <- b[ix, "fdur"] + b[ix-1, "fdur"] # add duration of fixations
	b[ix-1, "sacdur"] <- b[ix, "sacdur"] + b[ix-1, "sacdur"] # add duration of saccades
	b[ix-1, "x_component"] <- b[ix, "x_component"] + b[ix-1, "x_component"] # add up x components of saccades
	b[ix-1, "y_component"] <- b[ix, "y_component"] + b[ix-1, "y_component"] # add up y components of saccades
	b[ix-1, "peakvel"] <- b[ix + idxs[,1] - 2, "peakvel"] 		# set to argmax velocity of saccades
	b[ix-1, "xampl"] <- b[ix + idxs[,2] - 2, "xampl"] 	# set to argmax xampl of saccades		
	b[ix-1, "yampl"] <- b[ix + idxs[,3] - 2, "yampl"] 	# set to argmax yampl of saccades		
	b[ix-1, "isModified"] <- TRUE

	a[,relvars] <- b[,relvars]
	a$isModified <- b$isModified
	a <- a[!vshrtfix,] # remove short fixations
}
return(a)
}
