`saccpar` <-
function(bsac) 
{
#-------------------------------------------------------------------
#
#  FUNCTION saccpar.m
#
#  (Version 1.0, 01 AUG 05)
#
#-------------------------------------------------------------------
#
#  INPUT: binocular saccade matrix from FUNCTION binsacc.m
#
#  sac[,1:14)       binocular microsaccades 
#
#  OUTPUT:
#
#  sac[,1:9)        parameters avergaed over left and right eye data
#
#---------------------------------------------------------------------
if (nrow(bsac) > 0) {
    sacr = bsac[, 1:7]
    sacl = bsac[, 8:14]

    # 1. Onset
    a <- pmin(sacr[, 1], sacl[, 1])

    # 2. Offset
    b <- pmax(sacr[, 2], sacl[, 2])

    # 3. Duration
    DR <- sacr[, 2] - sacr[, 1] + 1
    DL <- sacl[, 2] - sacl[, 1] + 1
    D <- (DR + DL)/2

    # 4. Delay between eyes
    delay <- sacr[,1] - sacl[,1]

    # 5. Peak velocity
    vpeak <- (sacr[,3] + sacl[,3])/2

    # 6. Saccade distance
    dist <- (sqrt(sacr[,4]^2 + sacr[,5]^2) + sqrt(sacl[,4]^2 + sacl[,5]^2))/2
    angle1 <- atan2((sacr[,5] + sacl[,5])/2, (sacr[,4] + sacl[,4])/2)

    # 7. Saccade amplitude
    ampl <- (sqrt(sacr[,6]^2 + sacr[,7]^2) + sqrt(sacl[,6]^2 + sacl[,7]^2))/2
    angle2 <- atan2((sacr[,7] + sacl[,7])/2, (sacr[,6] + sacl[,6])/2)

    sac <- cbind(a, b, D, delay, vpeak, dist, angle1, ampl, angle2)
	sac <- data.frame(sac)
	names(sac) <- c('t0', 't1', 'D', 'delay',  'vpeak',  'dist', 'angle1', 'ampl', 'angle2')

} else {
    sac = NULL
}
sac 
}

