`treatBadData` <- function(bad, fpos, rightSentenceBorder, PIX_PER_CHAR)
{#-------------------------------------------------------------------
#
# treatment of 'invalid fixations' (e.g. blinks) outside of sentence boundaries
#-------------------------------------------------------------------
#
#  INPUT:
#
#  bad              index vector containing numeric (not logical!) indices of 'bad' fixation positions
#  fpos             vector containing mean fixation positions
#  rightSentenceBorder right sentence border
#  PIX_PER_CHAR     pixels per character
#
#  OUTPUT:
#
#  nsac_start       number of first valid sac./ fix.
#  nsac_end         number of last valid sac./ fix.
#  excludeSent      set to 1 if sentence is excluded from analyses
#---------------------------------------------------------------------

excludeSent  <-  0  # do not exclude the whole sentence from analyses
nsac_start <- 1
nsac_end <- 1
havePastRightBorder <- FALSE
if (length(bad)>0) {     # no 'invalid fixations'?  then everything is o.k.
    # remove these fixations
    for (i in 1:length(bad)) {
        # invalid fixation(s) RIGHT of the sentence
        if (fpos[bad[i]] > rightSentenceBorder + PIX_PER_CHAR) {
			if (!havePastRightBorder ) {
				havePastRightBorder <- TRUE
				nsac_end <- bad[i]-1  # bad(i) is the first invalid fix.
			}
            # delete ALL fixations after the eyes have passed the right sentence border
        } else {    # invalid fixation(s) LEFT of the sentence
            if (bad[i] == 1) {   # if the first fix. in a sentence is invalid, start with the second fix
                nsac_start <- 2
            } else {
                excludeSent <- 1   # exclude whole sentence from analyses
            }
        }
    } # for i in 1:length(h)
} # if bad
if (nsac_start >= nsac_end) {
	excludeSent <- 1   # exclude whole sentence from analyses
}
res <- list(nsac_start=nsac_start, nsac_end=nsac_end, excludeSent=excludeSent)
res
}
