`wordLetterPos` <- function(fpos, sWL, PIX_PER_CHAR, wbr_all=NULL, wbl_all=NULL)
{
#-------------------------------------------------------------------
#
#  translate fixation position [pix] to word and letter position 
#  we use fractional letters to avoid (accumulation of) rounding errors
#
#  Matlab Version 1.0, 06 APRIL 06 by Antje Nuthmann
#  combination, simplification and re-implementation of former wordpos.m & letterpos(_b).m
#
#  Matlab Version 2.0, October 08 by Jochen Laubrock
#  complete rewrite, simplification, now uses floating-point letter positions instead of integer binning 
#  (to avoid accumulation of rounding errors in later processing stages, e.g., calculation of saccade lengths)
#  interface remains unchanged, so this shouldn't break anything
#
#  R version 2.0, January 09 by Jochen Laubrock
#  changed interface to no longer pass satz and sID, because only the current sentence's word lengths are needed
#
# TODO: 
# 	make it more generic
# 	remove dependency on PIX_PER_CHAR
#-------------------------------------------------------------------
#
#  INPUT:
#
#  fpos             	fixation position (relative to sentence onset)
#  sWL				 		vector containing word length information (in letters) for the current sentence
#  PIX_PER_CHAR     		number of pixels per character (how many pixel make up a character)
#  wbr_all				optional vector containing left word boundaries for all words 
#  wbl_all				optional vector containing right word boundaries for all words
# note: if wbl_all and wbr_all are provided, sWL is not used
#
#  OUTPUT:
#
#  res$w                word fixated (number of the word in the sentence which was fixated)
#  res$l                letter fixated (as integer)
#---------------------------------------------------------------------

# v2.0 needs some more testing (some SHK want to do it?)
# (procedure)
# (1) find pixel equivalent to word and letter boundaries
# (2) find fixated word, and scale fixated letter within word
# e.g., fix@513 pix, word 6 wb@{480,535}, PixPerChar=11 ->  fixPos@{word=6, ltr=(513-480)/11
# (3) treat special cases
# (4) write tests to compare v1 and v2 output

w <- NA
wl1 <- NA
OUTSIDE <- FALSE

# prepare borders (in pixel) for words and character
wl <- sWL # vector 'wl' contains the lengths of the words of sentence with ID 'sID'
wl[-1] <- wl[-1] + 1  # add a character space to all but the first word of the sentence 
		# in the literature, the empty space between words is assigned to the word to the right, i.e., the following word                        
		# wl[-1] means drop first element, so it is a short form for wl[2:length(wl)]
if (is.null(wbr_all)){
	wbr_all <- cumsum(wl) * PIX_PER_CHAR  # for each word, the right word border [pixel] is determined relative to the beginning of the sentence
}
if (is.null(wbl_all)){
wbl_all <- c(0, wbr_all[1:length(wbr_all)-1])  # compute left word boundaries in pixels from offset
		# NB: to emulate Antje, we would have to start with 1 here, but it think it is nicer to start with 0
}
# since we have fixpos relative to sentence onset, we can immediately determine the fixated word 
toTheLeft <- which(fpos > wbl_all)
toTheRight <- which(fpos <= wbr_all)

# deal with fixations left of sentence
if (length(toTheLeft) == 0) {
	OUTSIDE <- TRUE
	# w <- 1
	# w1 <- 1
	# l <- wbl_all[1]  # assign fixation position to first letter of first word, as Antje did. NB: should we really do this?
	# alternatively
	w <- NA
	w1 <- NA
	l <- NA
}

# deal with fixations right of sentence
if (length(toTheRight) == 0) {
	OUTSIDE <- TRUE
	# w <- length(wbr_all)	
	# w1 <- length(wbr_all)
	# l <- wl[length(wl)] # assign fixation position to last letter of last word. NB: should we really do this?
	# alternatively
	w <- NA
	w1 <- NA
	l <- NA
}

# if in sentence, determine fixation postion in letters (fractional)
if (!OUTSIDE){
	w <- max(toTheLeft)
	w1 <- min(toTheRight)
	# given w, we can now compute the fixated letter l
	l <- (fpos - wbl_all[w]) / PIX_PER_CHAR
}

# just double check, this is probably unnecessary
if (!is.na(w) && !is.na(w1) && w != w1) { stop(sprintf('error in wordLetterPos: check : w==%f  !=  w1==%f for fpos at %f', w, w1, fpos))}

res <- list(w=w, l=l)
res

}

# BELOW: old code snippets
# # deal with fixations left of sentence
# OUTSIDE <- FALSE
# if (!is.finite(w)) {
# 	OUTSIDE <- TRUE
# 	w <- 1
# 	w1 <- 1
# 	l <- wbl_all[1]  # assign fixation position to first letter of first word, as Antje did. NB: should we really do this?
# 	# alternatively
# 	# w <- NA
# 	# l <- NA
# }
# 
# # deal with fixations right of sentence
# if (!is.finite(w1)) {
# 	OUTSIDE <- TRUE
# 	w <- length(wbr_all)	
# 	w1 <- length(wbr_all)
# 	l <- wl[length(wl)] # assign fixation position to last letter of last word. NB: should we really do this?
# 	# alternatively
# 	# w <- NA
# 	# l <- NA
# }


