plot_subject_hists <- function(data=d.all, whichVar=list(name="GZD", label="gaze duration", factorName="id"), TMAX = 800, NBREAKS=20, SAVEPLOT = FALSE, OUTPUT_FIG_PATH=".", Fmax=.15, subtxt = "", cols="black", type="density", ... )
# plot 1 histogram per factor
{
ix <- !is.na(data[, whichVar$name]) & data[, whichVar$name] <= TMAX
depvar <- data[ix, whichVar$name]
classvar <- data[ix, whichVar$factorName]

if(SAVEPLOT) {
	pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_by_", whichVar$factorName, "histogram.pdf", sep="")))
} else {
	quartz()
}
if (nchar(subtxt)==0) {
	subtxt <- paste("N =", length(subjhists), "lines, each representing one participant")
}
plot(1, type="n", xlim=c(0,TMAX), ylim=c(0, Fmax), xlab=whichVar$label, ylab="density", sub=subtxt, ...)

if (length(cols) < length(subjhists)) {cols <- seq(along=subjhists)}
if (type=="density") {
	subjdensity <- tapply(depvar, classvar, density)
	for (i in seq(along=subjdensity)) {
		h <- subjdensity[[i]]
		lines(subjdensity[[i]], col=cols[i])
	}
} else {
	for (i in seq(along=subjhists)) {
		subjhists <- tapply(depvar, classvar, hist, seq(0,TMAX,length.out=NBREAKS), plot=FALSE)
		h <- subjhists[[i]]
		lines(h$mids, h$density, col=cols[i], ...)
	}
}
if(SAVEPLOT) {dev.off()}

}

# some usage examples:
plot_subject_hists(data=d.all, whichVar=list(name="GZD", label="gaze duration", factorName="id"), TMAX = 800, SAVEPLOT = FALSE, OUTPUT_FIG_PATH=".", main="interindividual variance")

plot_subject_hists(data=d.all, whichVar=list(name="SFD", label="single fixation duration", factorName="id"), TMAX = 600, SAVEPLOT = FALSE, OUTPUT_FIG_PATH=".", main="interindividual variance" )

palette(rainbow(7))
colors <- tapply(d.all$ageGroup, as.numeric(d.all$id), unique)
colors <- as.vector(colors) + 1
plot_subject_hists(data=d.all, whichVar=list(name="GZD", label="gaze duration", factorName="id"), TMAX = 800, NBREAKS=30, SAVEPLOT = FALSE, OUTPUT_FIG_PATH = "/Users/jochen/Projects/presentations/warsaw/Reading/figs", Fmax=.012, cols=colors, lwd=.6, main="interindividual variance")
legend(500,.011, c("20+","30+","40+","50+","60+","70+"), lwd=1, col=unique(colors), bty="n")

colors <- c(1,5)
plot_subject_hists(data=d.all, whichVar=list(name="GZD", label="gaze duration", factorName="cwfw"), TMAX = 800, NBREAKS=40, SAVEPLOT = FALSE, OUTPUT_FIG_PATH = "/Users/jochen/Projects/presentations/warsaw/Reading/figs", Fmax=.007, cols=colors, lwd=2, main="effect of word type", subtxt="content vs. function words" )
legend(500,.005, c("content words","function words"), lwd=2, col=unique(colors), bty="n")
