# Program xIndex_fix.R
#
# demo: compute some useful indices
# 	monocular version
# 	see xIndex_fix_l3_logical.R for a binocular version
#
# Tranlated from Matlab into R S. Ohl 20/04/06
#
# - new duration and amplitude limits
# - compute new indices for long amplitudes
# - compute indices used in JEP:Gen (e.g., xR1st, xB1st)
# R. Kliegl 26/04/06
# adapted to n2bnd, 06/03/06 jochen laubrock
# extended by Sarah Risse, October 2006
#
# adapted to BIL2007, Sarah Risse, June 2007
# adapted to lesen III, jochen laubrock
# some changes 02/08 jochen laubrock (use list instead of array, use symbolic names)
# further cleanup, and conversion to logical indices 02/09 jochen laubrock
# monocular demo with Albrecht's data, 03/10, jochen laubrock

 
rm(list=ls())

projectDir <- "~/Projects/work/eyelinkTest/AAA"
basedataDir <- file.path(projectDir, "daten", "Experiment")
seqDir <- file.path(basedataDir, "fix_seqs")
dataDir <- seqDir

wbfile <- file.path(basedataDir, 'info', 'AAA_wb.dat')
ifile <- file.path(seqDir, "AAA_fix_RIGHT.rda")
# output file for logical indices
ofile <- file.path(dataDir, "AAA_fix_index_logical.rda") 
# output file for numerical indices 
ofile1 <- file.path(dataDir, "AAA_fix_index_numeric.rda")


load (ifile)  # right-eye fixations
dim(a) # 18089    93

frstfix <- list()
lastfix <- list()

# initialize lists
targetlistnames <- c('shrt', 'long', 'laout', 'lain', 'mcout', 'mcin', 'first', 'last', 'frstfix', 'lastfix', 'firstp', 'morep',
 'vFirst', 'vLast', 'valid', 'vFix', 'vFirstp', 'vMorep', 'x')
for (i in targetlistnames) {
	eval(parse(text = paste(i, '<- list()')))
}

xB <- list()
xB1st <- list()

# Index for short or long fixations 
shrt <- a$dur <= 50  # modified 26/04/06
long <- a$dur >= 750  # modified 26/04/06

# Index for long amplitudes, added 26/04/06
laout <- a$ao >= 25
lain <- a$ao >= 25

# Index for within-letter sacc
	# initialize with FALSE
mcout <- floor(a$ao) == 0
mcin <- floor(a$ao1) == 0

# Index for fixations on first and last words
first <- a$wn == 1
last <- a$wn == a$nw

# Index for sentence boundaries
sid <- a$id * 1000 + a$sn  # Recompute because records were deleted
six <- parseq(sid)           # Start and end of fix for each sentence

# Index for first and last fixations of sentences
dummy <- rep(FALSE, nrow(a))
frstfix <- dummy
lastfix <- dummy
frstfix[six[,1]] <- TRUE
lastfix[six[,2]] <- TRUE

firstp_h <- c()				# Firstpass fixations = 1; Secondpass fixations = 0
for (j in 1:nrow(six))	     
{
    ix <- six[j, 1]
    iy <- six[j, 2]
    fpf <- firstpass(a[ix:iy, "wn"]) # One sentence at a time
    firstp_h <- c(firstp_h, fpf)
    ix <- ix + iy
}
firstp <- firstp_h == TRUE	# Keep indices for firstpass fix
morep <- firstp_h == FALSE	# indices for second and further pass reading fixations


# replace NAs with FALSE to enable logical operations 
mcout[is.na(mcout)] <- FALSE
mcin[is.na(mcin)] <- FALSE
laout[is.na(laout)] <- FALSE
lain[is.na(lain)] <- FALSE

# Index for valid fix on First/Last words
vFirst <- first & !(shrt | long | mcin | mcout)
vLast <- last & !(shrt | long | mcin | mcout)

# Index for all valid fixations: Core set
aix <- rep(TRUE, nrow(a))
valid <- !(shrt | long | lain | laout | mcin | mcout | first | last | frstfix | lastfix)

# Index for valid fixations, irrespective of saccade amplitude
vFix <- !(shrt | long | first | last | frstfix | lastfix)

# Index for valid first-pass fixations
vFirstp <- valid & firstp


# Index for valid second-(and more)pass fixations
vMorep <- valid & morep


# # Index used in KNE2006: Identify single fixations on same word in both eyes ...
# # ... preceded and followed by a forward saccade in firstpass reading
# #
# # here we have only monocular recording, so cannot the first criterion
# #
# # note: lines below assume an equal number of rows (fixations) in left and right eye
# # if we give up this constraint, we need to re-work these indexes (up to xBinocAll)
# stopifnot(nrow(a[[1]]) == nrow(a[[2]]))
# x[[1]] <- a[[1]]$wn > a[[1]]$wn1.x  &  a[[1]]$wn < a[[1]]$wn2.x  &  a[[1]]$wn==a[[2]]$wn
# x1st[[1]] <- vFirstp[[1]] & x[[1]]
# 
# x[[2]] <- a[[2]]$wn>a[[2]]$wn1.x & a[[2]]$wn<a[[2]]$wn2.x & a[[1]]$wn==a[[2]]$wn
# x1st[[2]] <- vFirstp[[2]] & x[[2]]
# 
# 
# xB1st <- x1st[[1]] & x1st[[2]]
# xB <- x[[1]] & x[[2]]
# 
# # Index for binocular fixations
# # (both eyes are on the same word, all fixations (also invalids) included)
# #
# xBinocAll <- a[[1]]$wn == a[[2]]$wn

# # name lists
# for (i in targetlistnames) {
# 	eval(parse(text = paste('names(', i, ')', '<- c("LEFT", "RIGHT")' )))
# }
# savevars <- c(targetlistnames, 'six', 'xB1st', 'xB', 'xBinocAll')
# savevars <- c('shrt', 'long', 'laout', 'lain', 'mcout', 'mcin', 'first', 'last','frstfix', 'lastfix', 'firstp', 'morep', 
#      'vFirst', 'vLast', 'valid', 'vFix', 'vFirstp', 'vMorep', 'x', 'x1st', 'six', 'xB1st', 'xB', 'xBinocAll')

# x adapted to monocular gaze
x <- a$wn > a$wn1.x  &  a$wn < a$wn2.x  


savevars <- c(targetlistnames, 'six')
save(list=savevars, file=ofile)

# convert to numerical indices
# list ixn contains numerical versions of all indices
ixn <- vector("list", length(savevars))
names(ixn) <- savevars

for (var in setdiff(savevars, "six")) {
	eval(parse(text = paste('ixn[["', var, '"]] <- which(', var, ')', sep="")))
}
ixn$six <- six

save(ixn, file=ofile1)