# Program xIndex_fix.R
#
# Based on ~/documents/projects/EyeMoves/Sven_Ohl/xMonocDef.R
# Tranlated into R
# S. Ohl 20/04/06
#
# - new duration and amplitude limits
# - compute new indices for long amplitudes
# - compute indices used in JEP:Gen (e.g., xR1st, xB1st)
# R. Kliegl 26/04/06
# adapted to n2bnd, 06/03/06 jochen
# extended by Sarah Risse, October 2006
#
# adapted to BIL2007, Sarah Risse, June 2007
#
# adapted to lesen III, jochen 
# some changes 02/08 jochen laubrock (use list instead of array, use symbolic names)
# further cleanup, and conversion to logical indices 02/09 jochen laubrock
 
rm(list=ls())

projectDir <- "~/Projects/work/eyelinkTest/lesen III"
dataDir <- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R")

ifiles <- c( 	file.path(dataDir, "PSCII_l3_fix_LEFT_binoc.rda"), 
				file.path(dataDir, "PSCII_l3_fix_RIGHT_binoc.rda")	)
# output file for logical indices
ofile <- file.path(dataDir, "PSCII_l3_fix_index_logical.rda") 
# output file for numerical indices 
ofile1 <- file.path(dataDir, "PSCII_l3_fix_index_numeric.rda")

load (ifiles[1])  # left-eye fixations
aL<-a
dim(aL) # 110759 96
load (ifiles[2])  # right-eye fixations
aR<-a
dim(aR) # 110759 96

a<- list(aL, aR)
names(a) <- c("LEFT", "RIGHT")
rm(aL)
rm(aR)
gc()


frstfix <- list()
lastfix <- list()

# initialize lists
targetlistnames <- c('shrt', 'long', 'laout', 'lain', 'mcout', 'mcin', 'first', 'last', 'frstfix', 'lastfix', 'firstp', 'morep',
 'vFirst', 'vLast', 'valid', 'vFix', 'vFirstp', 'vMorep', 'x', 'x1st')
for (i in targetlistnames) {
	eval(parse(text = paste(i, '<- list()')))
}

xB <- list()
xB1st <- list()

for (i in 1:2) { # loop over eyes
	# Index for short or long fixations 
	shrt[[i]] <- a[[i]]$dur <= 50  # modified 26/04/06
	long[[i]] <- a[[i]]$dur >= 750  # modified 26/04/06

	# Index for long amplitudes, added 26/04/06
	laout[[i]] <- a[[i]]$ao >= 25
	lain[[i]] <- a[[i]]$ao >= 25

	# Index for within-letter sacc
		# initialize with FALSE
	mcout[[i]] <- floor(a[[i]]$ao) == 0
	mcin[[i]] <- floor(a[[i]]$ao1) == 0
	
	# Index for fixations on first and last words
	first[[i]] <- a[[i]]$wn == 1
	last[[i]] <- a[[i]]$wn == a[[i]]$nw

	# Index for sentence boundaries
	sid <- a[[i]]$id * 1000 + a[[i]]$sn  # Recompute because records were deleted
	six <- parseq(sid)           # Start and end of fix for each sentence

	# Index for first and last fixations of sentences
	dummy <- rep(FALSE, nrow(a[[i]]))
	frstfix[[i]] <- dummy
	lastfix[[i]] <- dummy
	frstfix[[i]][six[,1]] <- TRUE
	lastfix[[i]][six[,2]] <- TRUE

	firstp_h <- c()				# Firstpass fixations = 1; Secondpass fixations = 0
	for (j in 1:nrow(six))	     
	{
	    ix <- six[j, 1]
	    iy <- six[j, 2]
	    fpf <- firstpass(a[[i]][ix:iy, "wn"]) # One sentence at a time
	    firstp_h <- c(firstp_h, fpf)
	    ix <- ix + iy
	}
	firstp[[i]] <- firstp_h == TRUE	# Keep indices for firstpass fix
	morep[[i]] <- firstp_h == FALSE	# indices for second and further pass reading fixations
	
	
	# replace NAs with FALSE to enable logical operations 
	mcout[[i]][is.na(mcout[[i]])] <- FALSE
	mcin[[i]][is.na(mcin[[i]])] <- FALSE
	laout[[i]][is.na(laout[[i]])] <- FALSE
	lain[[i]][is.na(lain[[i]])] <- FALSE
	
	# Index for valid fix on First/Last words
	vFirst[[i]] <- first[[i]] & !(shrt[[i]] | long[[i]] | mcin[[i]] | mcout[[i]])
	vLast[[i]] <- last[[i]] & !(shrt[[i]] | long[[i]] | mcin[[i]] | mcout[[i]])
	
	# Index for all valid fixations: Core set
	aix <- rep(TRUE, nrow(a[[i]]))
	valid[[i]] <- !(shrt[[i]] | long[[i]] | lain[[i]] | laout[[i]] | mcin[[i]] | mcout[[i]] | first[[i]] | last[[i]] | frstfix[[i]] | lastfix[[i]])

	# Index for valid fixations: irrespective of saccade amplitude
	vFix[[i]] <- !(shrt[[i]] | long[[i]] | first[[i]] | last[[i]] | frstfix[[i]] | lastfix[[i]])

	# Index for valid first-pass fixations
	vFirstp[[i]] <- valid[[i]] & firstp[[i]]


	# Index for valid second-(and more)pass fixations
	vMorep[[i]] <- valid[[i]] & morep[[i]]
} # loop over eyes

# Index used in KNE2006: Identify single fixations on same word in both eyes ...
# ... preceded and followed by a forward saccade in firstpass reading
#
# note: lines below assume an equal number of rows (fixations) in left and right eye
# if we give up this constraint, we need to re-work these indexes (up to xBinocAll)

x[[1]] <- a[[1]]$wn > a[[1]]$wn1.x  &  a[[1]]$wn < a[[1]]$wn2.x  &  a[[1]]$wn==a[[2]]$wn
x1st[[1]] <- vFirstp[[1]] & x[[1]]

x[[2]] <- a[[2]]$wn>a[[2]]$wn1.x & a[[2]]$wn<a[[2]]$wn2.x & a[[1]]$wn==a[[2]]$wn
x1st[[2]] <- vFirstp[[2]] & x[[2]]


xB1st <- x1st[[1]] & x1st[[2]]
xB <- x[[1]] & x[[2]]

# Index for binocular fixations
# (both eyes are on the same word, all fixations (also invalids) included)
#
xBinocAll <- a[[1]]$wn == a[[2]]$wn

# name lists
for (i in targetlistnames) {
	eval(parse(text = paste('names(', i, ')', '<- c("LEFT", "RIGHT")' )))
}
savevars <- c(targetlistnames, 'six', 'xB1st', 'xB', 'xBinocAll')
# savevars <- c('shrt', 'long', 'laout', 'lain', 'mcout', 'mcin', 'first', 'last','frstfix', 'lastfix', 'firstp', 'morep', 
#      'vFirst', 'vLast', 'valid', 'vFix', 'vFirstp', 'vMorep', 'x', 'x1st', 'six', 'xB1st', 'xB', 'xBinocAll')
	
save(list=savevars, file=ofile)

# convert to numerical indices
# list ixn contains numerical versions of all indices
ixn <- vector("list", length(savevars))
cnt <- 1
for (var in targetlistnames) {
	for (i in 1:2) {
		# "ixn[[ 1 ]][[ 1 ]] <- which( shrt [[ 1 ]])" etc.
		eval(parse(text = paste('ixn[[', cnt, ']][[', i, ']] <- which(', var, '[[', i, ']])'))) 
	}
	names(ixn)[cnt] <- var
	cnt = cnt + 1
}
for (var in setdiff(setdiff(savevars, 'six'), targetlistnames)) {
	# "ixn[[ 21 ]] <- which( xB1st )" etc.
	eval(parse(text = paste('ixn[[', cnt, ']] <- which(', var, ')'))) 
	names(ixn)[cnt] <- var
	cnt <- cnt + 1
}
ixn[[cnt]] <- six
names(ixn)[cnt] <- 'six'

save(ixn, file=ofile1)