# Program xSetup1_or  
#
# prepare for distributed-processing analyses
# 
# enhance readability and general cleanup, jl 02/09
# added predictabilites jl 07/08
# Jochen Laubrock (jl) 08/06, version for oral reading, eyelink 1000
# james ong added a clever speedup
# most work done earlier by first version provided by R. Kliegl 30/07/05


rm(list=ls())
library('eyetrackR')

projectDir <- "~/Projects/work/eyelinkTest/lesen III"
psc2wbfile <-  file.path(projectDir,"/PSC2_wb_l3.dat")
dataDir <- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R")

ifile<- c( 	file.path(dataDir, "PSCII_allsac_all_LEFT.seq"),
			file.path(dataDir, "PSCII_allsac_all_RIGHT.seq"),
			file.path(dataDir, "PSCII_allsac_all_BOTH.seq") 	)

ofile <- c( 	file.path(dataDir, "PSCII_l3_fix_LEFT_binoc.rda"), 
				file.path(dataDir, "PSCII_l3_fix_RIGHT_binoc.rda"), 
				file.path(dataDir, "PSCII_l3_fix_BOTH_binoc.rda")	)

# ifile<- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R/PSCII_allsac_all_RIGHT.seq")
# ofile <- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R/psc2_l3_fix.rda")

# ======================================================
# =             LOAD AND PREPARE CORPUS                =
# ======================================================
# get word info
# loading PSC2.DWDS corpus
data(PSC2.DWDS)


# it is in principle a good idea to have a file containing word boundaries in pixels on the subject screen
# 
# load info about word boundaries (this is prepared by matlab, ~/Projects/work/eyelinkTest/lesen III/matlab/functions/wb_l3.m)
# NOTE: this is dangerous if the content of the external file is not known to be valid 
# make sure that it is correct!!
HAVE_WB_FILE <- FALSE
if (HAVE_WB_FILE){
	wbvarnames=c("sn","nw","wn","x0","x1","nchr", "wb")
	PSC2.wb <- read.table(psc2wbfile, header = FALSE, sep = "\t", dec = ".", col.names=wbvarnames, na.strings = "NaN", colClasses = NA, nrows = 1230)
	PSC2.wb$sn<-as.factor(PSC2.wb$sn)

	# PSC2.DWDS$nchr <- PSC2.wb$nchr
	# PSC2.DWDS$wb <- PSC2.wb$wb
	PSC2.DWDS$x0 <- PSC2.wb$x0
	PSC2.DWDS$x1 <- PSC2.wb$x1
	rm("PSC2.wb")
} else { # assume fixed offset and correct PIX_PER_CHAR; compute pixel values
	data(parms1)
	x0 <- vector("numeric", length=nrow(PSC2.DWDS))
	x1 <- vector("numeric", length=nrow(PSC2.DWDS))
	numSn <- as.numeric(PSC2.DWDS$sn)
	cnt <- 1
	for ( i in unique(numSn)) {
		six <- numSn == i
		wix <- cnt:(cnt + sum(six) - 1)
		x1[wix] <- LEFT_SENTENCE_OFFSET - PIX_PER_CHAR + cumsum(PSC2.DWDS[six,"nchr"]) * PIX_PER_CHAR
		x0[wix] <- c(LEFT_SENTENCE_OFFSET - PIX_PER_CHAR + .01, x1[ wix[ 1:(length(wix)-1) ] ] + .01)
		cnt <- cnt + sum(six)
	}
	PSC2.DWDS$x0 <- x0
	PSC2.DWDS$x1 <- x1
	# the above assumes that we also add a blank space in front of the first word (strange, because there is no inter-word space)
	# if we want to treat the first word special (no blank in front of it), uncomment next line
	# PSC2.DWDS[PSC2.DWDS$wn==1,"x0"] <- LEFT_SENTENCE_OFFSET 
}

attach(PSC2.DWDS) 
corpus <- data.frame(cbind(sn, wn, wb, 1/nlett1, 1/nchr, f, pred, bifr, trifr, cwfw))
cidw <- as.numeric(sn) * 100 + wn
names(corpus)[4] <- 'inv_nlett1'
names(corpus)[5] <- 'inv_nchr'
detach(PSC2.DWDS)

nrow(corpus)

#james: turn 1552 rows to 16013 rows + 2 buffer rows for word n+1 and n+2
conv_corpus <- data.frame(matrix(data=NA, ncol=ncol(corpus), nrow=max(cidw)+2))
names(conv_corpus) <- c('sn', 'wn', 'wb', 'inv_nlett1', 'inv_nchr', 'f', 'p', 'bifrq', 'trifrq', 'cwfw') 
# names(conv_corpus) <- names(corpus)
# names(conv_corpus) <- c('sn', 'wn', 'wb', '1/nchr', 'f', 'bifrq', 'trifrq', 'tp', 'x') 
conv_corpus[cidw,] <- corpus
rm(corpus)


# ======================================================
# =             GET AND PREPARE FIXATIONS              =
# ======================================================
nfiles <- length(ifile)
aa <- list(nfiles)

for (fileCnt in 1:nfiles)
	{
	# GET FIXATIONS
	seq <- read.table(ifile[fileCnt])  # Left or right eye
	names(seq) <- c("id","sid","nw","wn","let","fdur","sacdur","peakvel", "xampl", "yampl", "x_component", "y_component", "fpos0", "fpos1", "fpos", "firstLastInSentence")

	ix <- which(seq[,"fdur"]>=50  & seq[,"fdur"]<=1000)

	op <- par(mfcol = c(1,3))
	sacdx <- seq[, "xampl"]
	sacdxix <- seq[ix, "yampl"]
	fixdur <- seq[ix, "fdur"]
	hist(sacdxix,breaks=500, xlim=c(-10,20))
	hist(sacdxix[sacdxix<=3], breaks=150, xlim=c(-1,3))
	hist(fixdur,500)

	# SCREEN DATA, NOTE: ONLY ENTIRE SENTENCES OR IDs CAN BE REMOVED
	# Removing fixations within sentences destroys sequence!
	# here, nothing is removed

	# Remove sentences with max 2 fixated words
	sid <- seq[,1,1] * 1000 + seq[,2,1]
	six <- parseq(sid)

	badsent <- which(six[,3]<=2) 
	badfix2 <- vector(mode="numeric", length=sum(six[badsent,2]-six[badsent,1]+1))	
	if (length(badsent)>0) {
		cnt <- 1
		for (i in 1:length(badsent))
		{
			badix <- six[badsent[i],1]:six[badsent[i],2]
			badfix2[cnt:(cnt+length(badix)-1)] <- badix
			cnt <- cnt + length(badix)
		}
		seq <- seq[-badfix2, , ]
		nrow(seq)
	}
	aa[[fileCnt]] <- seq
}

# ======================================================
# =         remove ultra-short fixations               =
# ======================================================

# TODO: the part below is an ugly hack, we should be able to get rid of it by treating potential problems earlier (saccade detection?)

# remove ultra-short fixations (this needs to be done binocularly, unfortunately)
# source("~/Projects/R library/potsdam/testing/removeShortFixations.R")
MIN_DUR <- 14	# minimum duration of a fixation to keep
aa2 <- removeShortFixationsBinoc(aa[[1]], aa[[2]], MIN_DUR)
# wir bekamen weiter unten (xIndex) ein Problem, da nun aL und aR bei monokulaerem removeShortFixations nicht mehr die gleiche Zeilenzahl hatten 
# das laesst sich wohl am leichtesten lösen, indem man hier noch mal binokuläre Sakkaden fordert

# hack to make sure that left and right eye's fixations are the same serial position:
# remove all sentences for which unlikelyfix==TRUE (about 1 in 10000)
# in these cases, left and right eye fixation duration of very short fixations differed by more than DCRIT=30 ms
cnt=1; for (element in aa2) {aa2[[cnt]] <- element[!element$unlikelyfix,]; cnt=cnt+1}
# the above is equivalent to
# aamod2[[1]] <- aamod2[[1]][!aamod2[[1]]$unlikelyfix,])
# aamod2[[2]] <- aamod2[[2]][!aamod2[[2]]$unlikelyfix,])
aa2[[nfiles]] <- removeShortFixations(aa[[nfiles]], MIN_DUR)	

rm(list=c('aa', 'seq'))

# ======================================================
#     prepare for distributed processing analyses      =
# ======================================================
for (fileCnt in 1:nfiles) {
	a <- aa2[[fileCnt]]
	nrow(a)
	a <- a[!is.na(a$wn),]
	a <- a[,c(1:6, 11)] 
	nrow(a)

	
	# INCLUDE INFORMATION ABOUT FIXATED AND NEIGHBORING WORDS
	# a[,8:55] <- NA
	# names(a) <- c('id',    'sn',  'nw', 
	#               'wn',   'let', 'dur', 'ao', 'dir', 'o', 'l',   'f',  
	#               'wn1',                                  'l1',  'f1',  
	#               'wn2',                                  'l2',  'f2',  
	#               'wn3',                                  'l3',  'f3',  
	#               'wn4',                                  'l4',  'f4',  
	#               'wn1.x','let1','dur1','ao1','dir1','o1','l1.x','f1.x',
	#               'wn2.x','let2','dur2','ao2','dir2','o2','l2.x','f2.x',
	#               'wn3.x','let3','dur3','ao3','dir3','o3','l3.x','f3.x',
	#               'wn4.x','let4','dur4','ao4','dir4','o4','l4.x','f4.x')

	a[,7:91] <- NA
	names(a) <- c('id',    'sn',  'nw', 
	              'wn',   'let', 'dur', 'ao', 'dir', 'o', 'l',   'f',   'p',   'bi',   'tri',   'cwfw', 
	              'wn1',                                  'l1',  'f1',  'p1',  'bi1',  'tri1',  'cwfw1',
	              'wn2',                                  'l2',  'f2',  'p2',  'bi2',  'tri2',  'cwfw2',
	              'wn3',                                  'l3',  'f3',  'p3',  'bi3',  'tri3',  'cwfw3',
	              'wn4',                                  'l4',  'f4',  'p4',  'bi4',  'tri4',  'cwfw4',
	              'wn1.x','let1','dur1','ao1','dir1','o1','l1.x','f1.x','p1.x','bi1.x','tri1.x','cwfw1.x',
	              'wn2.x','let2','dur2','ao2','dir2','o2','l2.x','f2.x','p2.x','bi2.x','tri2.x','cwfw2.x',
	              'wn3.x','let3','dur3','ao3','dir3','o3','l3.x','f3.x','p3.x','bi3.x','tri3.x','cwfw3.x',
	              'wn4.x','let4','dur4','ao4','dir4','o4','l4.x','f4.x','p4.x','bi4.x','tri4.x','cwfw4.x')

	# Word numbers
	a[a$wn > 1,      "wn1"] <- a[a$wn > 1,      "wn"] - 1
	a[a$wn < a$nw ,  "wn2"] <- a[a$wn < a$nw,   "wn"] + 1
	a[a$wn > 2,      "wn3"] <- a[a$wn > 2,      "wn"] - 2
	a[a$wn < a$nw-1, "wn4"] <- a[a$wn < a$nw-1, "wn"] + 2

	didw  <- a$sn*100+a$wn       # didw = data id of word

	aCurrentNames <- c("o", "l", "f", "p", "bi","tri", "cwfw")
	aL1Names <- c("l1", "f1", "p1", "bi1", "tri1", "cwfw1")
	aL2Names <- c("l2", "f2", "p2", "bi2", "tri2", "cwfw2")
	aL3Names <- c("l3", "f3", "p3", "bi3", "tri3", "cwfw3")
	aL4Names <- c("l4", "f4", "p4", "bi4", "tri4", "cwfw4")
	corpusVars <- c("wb", "inv_nlett1", "f", "p", "bifrq", "trifrq", "cwfw")
	corpusVars2 <-corpusVars[2:length(corpusVars)]

# TODO: I think we should refrain from using "o" as "wb" intermediately--this has confused me several times. Why not add "wb" to a? Discuss this with the team (jl)

	a[, aCurrentNames] <- conv_corpus[didw, corpusVars]
	a[, aL1Names] <- conv_corpus[didw-1, corpusVars2]
	a[, aL2Names] <- conv_corpus[didw+1, corpusVars2]
	a[, aL3Names] <- conv_corpus[didw-2, corpusVars2]
	a[, aL4Names] <- conv_corpus[didw+2, corpusVars2]

	# GENERATE SEQUENCES OF FIVE FIXATIONS, I.E. TWO LAGS, TWO SUCS 
	dim1 <- nrow(a)
	fillnan <- rep(NA, 12)

	# Add lag-1 fixation (44:55)
	fromVars  <- c("wn", "let", "dur", "ao", "dir", "o", "l", "f", "p", "bi", "tri", "cwfw" )
	lagVars   <- c("wn1.x", "let1", "dur1", "ao1" , "dir1", "o1"  , "l1.x"  , "f1.x"  , "p1.x"  , "bi1.x" , "tri1.x", "cwfw1.x" )
	lag2Vars  <- c("wn3.x", "let3", "dur3", "ao3" , "dir3", "o3"  , "l3.x"  , "f3.x"  , "p3.x"  , "bi3.x" , "tri3.x", "cwfw3.x" )
	succVars  <- c("wn2.x", "let2", "dur2", "ao2" , "dir2", "o2"  , "l2.x"  , "f2.x"  , "p2.x"  , "bi2.x" , "tri2.x", "cwfw2.x" )
	succ2Vars <- c("wn4.x", "let4", "dur4", "ao4" , "dir4", "o4"  , "l4.x"  , "f4.x"  , "p4.x"  , "bi4.x" , "tri4.x", "cwfw4.x" )

	lag <- a[,fromVars]              # copy "a" into "lag"
	lag <- lag[-dim1,]               # delete last row of "lag"
	lag <- rbind(fillnan, lag)       # insert NaN at beginning = shift down!
	a[,lagVars] <- lag               # put lag=down shifted "a" next to "a"
	rm(lag)

	# Add suc-1 fixation (56:67)
	suc <- a[,fromVars]
	suc <- suc[-1,]
	suc <- rbind(suc, fillnan)	
	a[,succVars] <- suc			
	rm(suc)

	# Add lag-2 fixation (68:79)
	lag2 <- a[,lagVars]
	lag2 <- lag2[-dim1,]
	lag2 <- rbind(fillnan,lag2) 
	a[,lag2Vars] <- lag2
	rm(lag2)

	# Add suc-2 fixation (80:91)
	suc2 <- a[,succVars]
	suc2 <- suc2[-1,]
	suc2 <- rbind(suc2, fillnan)
	a[,succ2Vars] <- suc2
	rm(suc2)

	# Index for sentence boundaries
	sid <- a[,1]*1000+a[,2]  
	six <- parseq(sid)               # Start and n of fix for each sentence

	# Replace impossible lag and succ values with NaNs
	a[six[,1],c(lagVars, lag2Vars)] <- fillnan[1] 		#  First fix of sentence: no lag-1
	a[six[,1]+1,c(lag2Vars)] <- fillnan[1] 				#  Second fix of sentence: no lag-2
	a[six[,2],c(succVars, succ2Vars)] <- fillnan[1] 	#  Last fix of sentence: no suc-1
	a[six[,2]-1,c(succ2Vars)] <- fillnan[1] 			#  Last-1 fix of sentence: no suc-2


	# # Absolute letter position of fixations on line, replaced later with "o" 
	# wbnames <- c("o","o1","o2","o3","o4")
	# wbpos <- match(wbnames, names(a) )
	# #wbpos <- c(9, 29, 37, 45, 53)
	# lettpos <- wbpos-4
	# lengthpos <- wbpos+1
	# j<-0
	# for (i in wbpos )   
	# {   
	# 	j <- j+1
	#     a[,i] <- a[,i]+a[,i-4]         # "9" contains wb, "5" cont letter position
	# }  


	# (jl) more concise replacement for the above,  ok?
	# Absolute letter position of fixations on line, replaced later with "o" 
	wbnames <- c("o","o1","o2","o3","o4")
	lettnames <- c("let","let1","let2","let3","let4")
	lengthnames <- c("l","l1","l2","l3","l4")
	for (j in 1:length(wbnames))   
	{   
	    a[,wbnames[j]] <- a[,wbnames[j]] + a[,lettnames[j]]         
	}

	pix <- c("o2", "o", "o4", "o1")
	aos <- c("ao", "ao1", "ao2", "ao3")
	dirs <- c("dir", "dir1", "dir2", "dir3")
	for (j in 1:4)
	{   
	    a[,aos[j]]   <- a[,pix[j]] - a[,wbnames[j]] # Signed amplitude
	    a[,dirs[j]] <- sign(a[,aos[j]])          # Strip off sign and store dir
	    a[,aos[j]]   <- abs(a[,aos[j]])           # Absolute amplitude in ao
	}

	# a$dir4 <- sign(a$ao4)  # For n+2 fix use dX amplitude, not letter distance
	# a$ao4  <- abs(a$ao4)   #

	# summary(a[a$dir>0,]$ao,100)

	a$ao4 <- c(a$ao2[-1],NA)  # delete first row to get ao2 of subsequent fixation
	a$dir4 <- c(a$dir2[-1],NA)


	# Convert letter position to relative position within word:
	# 0 = space before word, .50 = middle letter, 1 = last character (could be punct.)
	# a[,wbpos] <- a[,lettpos]/(1/a[,lengthpos]+1)
	a[,wbnames] <- a[,lettnames] / ( 1 / a[,c("l", "l1.x", "l2.x", "l3.x", "l4.x")] + 1)

	# Formula looks funny because "l" is stored as reciprocal!

	save(a,file=ofile[fileCnt])
}