# Program xSetup2.2.dvmat_AAA.R
# experiment:  AAA
# Sarah Risse, November 2006
# Jochen Laubrock, November 2006
# Jochen Laubrock, December 2006  
# Jochen Laubrock, 2010 # adapted to AAA


# generates matrix on word level with all dependent variable measures
# for target region containing word n to word n+3
# all measures generated from rigth eye indices


rm(list=ls())
library(eyetrackR)

projectDir <- "~/Projects/work/eyelinkTest/AAA"
basedataDir <- file.path(projectDir, "daten", "Experiment")
seqDir <- file.path(basedataDir, "fix_seqs")

ifile1 <- file.path(seqDir, "AAA_fix_RIGHT.rda") # file containing right-eye fixations in distributed-processing format
ifile2 <- file.path(seqDir, "AAA_fix_index_logical.rda") 
ofile <- file.path(seqDir, "AAA_dvmat.rda") 
corpusFile  <- file.path(basedataDir, 'info', 'AAA_corpus.dat')

# load corpus to generate balanced design matrix d.all
corpus <- read.delim(corpusFile, header=TRUE) # corpus info

load(ifile1)
load(ifile2)

dim(a)

# add index for skipping prior (s1) and subsequent (s2) word
# 0=no skip, 1=skip
# Skipping word n-1 (s1) and word n+1 (s2)
a$s1 <- a$ao1 - (1/a$l1 +1) - (a$let+1)
a$s1[a$s1 >= 0] <- 1
a$s1[a$s1 <  0] <- 0
#
a$s2 <- a$ao-(1/a$l2+1)-(1/a$l-a$let)
a$s2[a$s2 <  1] <- 0
a$s2[a$s2 >= 1] <- 1

# generate index-variables in matrix a
# will be used to select valid fixations for later generation 
# of different dependent variables
#
# generate index-variable for valid fixations (0=unvalid, 1=valid) 
# (exclusion of: long/short durations, long incoming and
# outgoing saccade amplitudes (>=25), first/last word)
#
a$valid <- valid

# generate firstpass (1) and second-and morepass (2) index 
# ALL fixations (not only valids) considered
# 
a$pass <- 0
a$pass[firstp] <- 1
a$pass[morep] <- 2


# generate single fixation index (1=single fix, 0=more fix)
# (no matter whether this word is later regressed to, that is
# if it gets further fixations in second pass reading)
# sequence of two progressive saccades (incoming and outgoing)
# 
a$Sfix <- x

# generate first fixation index (1=first fix, 0=refixation)
# (both firstpass and secondpass reading considered)

# individual word INDEX for unreduced matrix a (see below for word index matrix for d.all)
wid_a <- 1e5 * a$id + 1e2 * a$sn + a$wn 
widx_a<-parseq(wid_a)

a$Ffix <- FALSE
a$Ffix[widx_a[,1]] <- TRUE 

# save original matrix with all additional fixation indices
a.idx <- a

(words <- nrow(corpus))
(subjects <- length(unique(a$id)))
str(corpus)

# save word information into d.all (one row per combination of subject and corpus word)
# corpusCols <- c("sn", "nw", "wn", "nlett1", "f", "bifr", "trifr", "pred", "npred", "cwfw", "word")
corpusCols <- c("sn", "nw", "wn", "nlett1", "nlett2", "nchr", "frq", "spoken12", "spoken34", "t0f", "durf", "t0m", "durm", "word")

# jochen: faster and simpler than the old solution:
d.all <- corpus[rep(1:words, subjects), corpusCols]

d.all$id <- rep(1:subjects, each=words)
# cosmetic: move id and word columnbs to front
colsToMoveToFront <- c("id", "word")
d.all <- d.all[, c(colsToMoveToFront, setdiff(names(d.all), colsToMoveToFront))]
str(d.all)
table(d.all$id)
d.all$sn <- as.numeric(d.all$sn)

# NOTE:
# d.all: is sorted from subj 1 sentence 1 word 1
# a.idx: is sorted from subj 1 first exp. sentence (i.e., 129)

# individual word index for matrix d.all (each word for each subject gets independent id)
wid_d.all <- 1e5*d.all$id + 1e2*d.all$sn + d.all$wn

d.all$spoken <- a[match(wid_d.all, wid_a), "spoken"]

# see above for similar index into a.idx: wid_a <- 1e5 * a$id + 1e2 * a$sn + a$wn

###############################################################
# FFD analogous to Rayner et al.
# all first fixation durations (single fixation cases included)
###############################################################
	# valid first pass first fixations
ffixix <- a$valid & a$pass==1 & a$Ffix 
a.Ffix <- a[ffixix,]
wid_a.Ffix <- 1e5 * a.Ffix$id + 1e2* a.Ffix$sn + a.Ffix$wn
match.Ffix <- match(wid_a.Ffix, wid_d.all)

d.all$FFDr <- NA
d.all$FFDr[match.Ffix] <- a.Ffix$dur

# FFD analogous to Potsdam-group (first of multiple)
# all first fixation durations without single fixation cases 

a.Ffix1 <- a.Ffix[ a.Ffix$Sfix == FALSE, ]
wid_a.Ffix1 <- 1e5 * a.Ffix1$id + 1e2 * a.Ffix1$sn + a.Ffix1$wn
match.Ffix1 <- match(wid_a.Ffix1,wid_d.all)

d.all$FFDp <- NA
d.all$FFDp[match.Ffix1] <- a.Ffix1$dur

# match single fixation durations from matrix a
# (single fixations in first pass reading)

a.Sfix <- a[a$Sfix & a$valid & a$pass==1,]
wid_a.Sfix <- 1e5 * a.Sfix$id + 1e2 * a.Sfix$sn + a.Sfix$wn
match.Sfix <- match(wid_a.Sfix, wid_d.all)

d.all$SFD <- NA
d.all$SFD[match.Sfix] <- a.Sfix$dur

# match gaze durations from matrix a
##jochen: tapply by words a sum over the first pass fixations in a; then match that sum to d.all
# first pass
fpix <- (a$pass==1 & (a$ao1 <= 25 | is.na(a$ao1)) & (a$ao <= 25 | is.na(a$ao)))

## filter / remove fixations before regressions, should not contribute to standard stats 
beforeRegrIx <- a$dir == -1 & a$wn==a$wn2.x & !is.na(a$wn) & !is.na(a$wn2.x)

	# gaze for each word in first pass (how to match this to d.all?)
gzafp <- tapply(a$dur[fpix & !beforeRegrIx], wid_a[fpix & !beforeRegrIx], sum) 
	# total fixation duration
ttota <- tapply(a$dur, wid_a, sum)

# match.Gz <- match(as.numeric(sort(unique(wid_d[fpix])), wid_d.all)
match.Gz <- match(as.numeric(dimnames(gzafp)[[1]]), wid_d.all)
d.all$haveFirstPass <- FALSE
d.all$haveFirstPass[match.Gz] <- TRUE

d.all$GZD <- NA
d.all$GZD[match.Gz] <- gzafp

# total viewing time
match.ttota <- match(as.numeric(dimnames(ttota)[[1]]), wid_d.all)
d.all$haveFixation <- FALSE
d.all$haveFixation[match.ttota] <- TRUE

d.all[match.ttota, "TVT"] <- ttota

# is there a problem here--for s1 we need incoming of first fixation, for s2 outgoing after last fixation
# in Matrix d, can be s1 and s2
# in Matrix a, change to s1.x and s2.x because fixation based
xFix_a <- match(unique(wid_a),wid_a)
xFix_d.all <- match(unique(wid_a),wid_d.all)
d.all[xFix_d.all,  "s1"] <- a[xFix_a, "s1"]
d.all[xFix_d.all,  "s2"] <- a[xFix_a, "s2"]


sid_d.all <- 1e3 * d.all$id + d.all$sn
sid_a <- 1e3 * a$id + a$sn

# not every participant provides data for each sentence
# thus, we can only set d.all for valid sentences per Ss to 1, set it to NA else
# use a match of wid_a to wid_d.all to set potential candidates to 1 
validSentenceIxInD.all <- match(unique(wid_a), wid_d.all)
d.all$SKP <- NA # set SKP to NaN if sentence was never fixated
d.all[validSentenceIxInD.all, "SKP"] <- 1 # set default for valid sentences to skipped in first pass
d.all$SKP[match.Gz] <- 0	# set to not skipped if word was fixated in first pass
#set first and last word to NA
d.all$SKP[d.all$wn==1] <- NA
d.all$SKP[d.all$wn==d.all$nw] <- NA


# number of first pass fixations
d.all$nfp <- NA
d.all[validSentenceIxInD.all, "nfp"] <- 0
nafp <- tapply(a$dur[fpix], wid_a[fpix], length) 
d.all$nfp[match.Gz] <- nafp
d.all$nfp[d.all$wn==1 | d.all$wn==d.all$nw] <- NA

# number of all fixations
d.all$nap <- NA
d.all[validSentenceIxInD.all, "nap"] <- 0
naf <- tapply(a$dur, wid_a, length) 
d.all$nap[match.ttota] <- naf
d.all$nap[d.all$wn==1 | d.all$wn==d.all$nw] <- NA

# number of second-and-further pass fixations
d.all$nsp <- d.all$nap - d.all$nfp

# ingoing saccade length (ISL)
# (equals a$ao1)
# outgoing saccade length (OSL)
# current measure: in letters (characters)wid_a.fp <- wid_a[fpix]

a.fp <- a[fpix,]
wid_a.fp <- wid_a[fpix]
widx_a.fp <- parseq(wid_a.fp)
match.Gz1 <- match(wid_a.fp[widx_a.fp[,1]], wid_d.all) # same values as match.Gz, but unsorted

#brauchen wir unten fuer
d.all$ISL <- NA # incoming saccade length
d.all$OSL <- NA # outgoing saccade length
d.all$dir1 <- NA # incoming direction
d.all$dir <- NA # outgoing direction
d.all$ISL[match.Gz1] <- a.fp$ao1[widx_a.fp[,1]]
d.all$OSL[match.Gz1] <- a.fp$ao[widx_a.fp[,2]]
d.all$dir1[match.Gz1] <- a.fp$dir1[widx_a.fp[,1]]
d.all$dir[match.Gz1] <- a.fp$dir[widx_a.fp[,2]]

# initial landing position (ILP)
# (o at position first fixation)
# current measure: letter position/word length (0.5=word midth)
# NB: explorative OVP and I-OVP plots suggest that something went wrong in xallsac, check !!
d.all$ILP<-NA
d.all$ILP[match.Ffix]<-a.Ffix$o

# RE-SORTING OF MATRIX d.all
# matrix d.all contains all words for each subject
#

save(d.all, a.idx, file=ofile)

################################################################
#	done with the real work 
# 	below: some checks whether assignments were correct
################################################################

showPlot = TRUE
if (showPlot) {
	library(gtools)
	oldpar <- par(mfrow=c(3,4))
	
	# check: do we get an iovp effect (yes, clearly)
#	nbins <- 11
#	cilp <- cut(d.all$ILP, nbins)
#	iovp<-aggregate(d.all$FFDp, list(cilp), mean, na.rm=T)
#	plot(iovp$x, type="b", xlab="ILP", ylab="FFDp")
#	axis(3, 1:nbins, levels(cilp))

	# preferred viewing location effect: readers tend to fixate a point somewhere between the beginning and the middle of a word
	ix <- !is.na(d.all$ILP); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(d.all$ILP[ix], 5)	
	pvp <- aggregate(d.all$ILP[ix], list(q), length)
	plot(pvp$x, type="b", xlab="ILP", ylab="frequency")
	axis(3, 1:length(levels(q)), levels(q))

	# i-ovp: longer fixation duration near word center
	ix <- !is.na(d.all$ILP); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(d.all$ILP[ix], 5)	
	iovp<-aggregate(d.all$SFD[ix], list(q), mean, na.rm=T)
	plot(iovp$x, type="b", xlab="ILP", ylab="FFDp")
	axis(3, 1:length(levels(q)), levels(q))
	
	# current word length effect
	ix <- !is.na(d.all$GZD); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(d.all$nlett1[ix], 5)	
	gzdL <- aggregate(d.all$GZD[ix], list(q), mean, na.rm=T)
	plot(gzdL$x, type="b", xlab="length", ylab="GZD")
	axis(3, 1:length(levels(q)), levels(q))
	
	#check for ISL and OSL:
	ix <- !is.na(d.all$ISL); 
	# q <- quantcut(d.all$ISL[ix], q=seq(0,1,by=0.1), na.rm=T)
	q <- cut(d.all$ISL[ix], 5)
	tAsFuncOfISL <- aggregate(d.all$FFDp[ix], list(q), mean, na.rm=T); 
	plot(tAsFuncOfISL$x, type="b", xlab="ISL", ylab="FFDp")
	axis(3, 1:length(levels(q)), levels(q))
	
	# skipping as function of frequency
	ix <- !is.na(d.all$SKP); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(log10(d.all$frq[ix]), 5)	
	skpF <- aggregate(d.all$SKP[ix], list(q), mean, na.rm=T)
	plot(skpF$x, type="b", xlab="word frequency", ylab="skipping probability")
	axis(3, 1:length(levels(q)), levels(q))

	# skipping as function of frequency, by subject
	ix <- !is.na(d.all$SKP); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(log10(d.all$frq[ix]), 5)	
	skpF <- tapply(d.all$SKP[ix], list(q, d.all$id[ix]), mean, na.rm=T)
	matplot(skpF, type="b", xlab="word frequency", ylab="skipping probability")
	axis(3, 1:length(levels(q)), levels(q))

	# duration as function of frequency, by subject
	ix <- !is.na(d.all$GZD); 
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(log10(d.all$frq[ix]), 5)	
	gzdFID <- tapply(d.all$GZD[ix], list(q, d.all$id[ix]), mean, na.rm=T)
	matplot(gzdFID, type="b", xlab="word frequency", ylab="gaze duration")
	axis(3, 1:length(levels(q)), levels(q))
	
	ix <- !is.na(d.all$frq)
	# q <- quantcut(d.all$OSL[ix], q=seq(0,1,by=0.1), na.rm=T)
	q <- cut(log10(d.all$frq[ix]), 5)
	oslF <- tapply(d.all$OSL[ix], list(q, d.all$id[ix]), mean, na.rm=T); 
	matplot(oslF, type="b", xlab="frequency", ylab="OSL")
	# oslF <- tapply(d.all$OSL[ix], q, mean, na.rm=T); 
	# plot(oslF, type="b", xlab="frequency", ylab="OSL")
	axis(3, 1:length(levels(q)), levels(q))
	
	# check: do we get a pvp effect (yes, but check: is this really pvp?) 
	# TODO: check nsp!!
	#	nbins <- 9
	#	cilp <- cut(d.all$ILP, nbins)
	q <- cut(d.all$ILP[ix], 5)	
	pvp <- aggregate(d.all$nap[ix] > 1 , list(q), mean, na.rm=T)
	plot(pvp$x, type="b", xlab="ILP", ylab="f(refix)")
	axis(3, 1:length(levels(q)), levels(q))

	# trivial check: is ISL higher if last word was skippped?
	ix <- !is.na(d.all$s1); 
	# q <- quantcut(d.all$ISL[ix], q=seq(0,1,by=0.1), na.rm=T)
	q <- d.all$s1[ix]
	ISLAsFuncOfs1 <- aggregate(d.all$ISL[ix], list(q), mean, na.rm=T); 
	plot(ISLAsFuncOfs1$x, type="b", xlab="s1", ylab="ISL")
	axis(3, 1:length(levels(q)), levels(q))

	# trivial check: is OSL higher if next word will be skippped?
	ix <- !is.na(d.all$s2); 
	# q <- quantcut(d.all$ISL[ix], q=seq(0,1,by=0.1), na.rm=T)
	q <- d.all$s2[ix]
	OSLasFuncOfS2 <- aggregate(d.all$OSL[ix], list(q), mean, na.rm=T); 
	plot(OSLasFuncOfS2$x, type="b", xlab="s2", ylab="OSL")
	axis(3, 1:length(levels(q)), levels(q))
	
	quartz()
	par(mfrow=c(3,3))
	for (nl in 4:12) {# preferred viewing location effect: readers tend to fixate a point somewhere between the beginning and the middle of a word
		ix <- !is.na(d.all$ILP) & d.all$nlett1==nl 
		# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
		q <- cut(d.all$ILP[ix], 8)	
		pvp <- aggregate(d.all$ILP[ix], list(q), length)
		plot(pvp$x, type="b", xlab="ILP", ylab="frequency",  main=paste('word length =', nl-1))}
		# axis(3, 1:length(levels(q)), levels(q))

	quartz()
	par(mfrow=c(3,3))
	for (nl in 4:10) {# iovp effect: readers tend to fixate a point somewhere between the beginning and the middle of a word
		ix <- !is.na(d.all$ILP) & d.all$nlett1==nl 
		# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
		q <- cut(d.all$ILP[ix], 6)	
		iovp <- aggregate(d.all$FFDp[ix], list(q), mean,  na.rm=TRUE)
		plot(iovp$x, type="b", xlab="relative landing position [bin]", ylab="first fixation duration [ms]",  main=paste('word length =', nl-1))
	}
	ix <- !is.na(d.all$ILP) & d.all$nlett1>=4 & d.all$nlett1<=13
	# q <- quantcut(d.all$ILP[ix], q=seq(0,1,by=0.2), na.rm=T)	
	q <- cut(d.all$ILP[ix], 10)	
	iovp <- aggregate(d.all$FFDp[ix], list(q), mean,  na.rm=TRUE)
	plot(iovp$x, type="b", xlab="relative landing position [bin]", ylab="first fixation duration [ms]",  main='IOVP (FFD), wl>=3 & wl<=12')

	q <- cut(d.all$ILP[ix], 10)	
	iovp <- aggregate(d.all$SFD[ix], list(q), mean,  na.rm=TRUE)
	plot(iovp$x, type="b", xlab="relative landing position [bin]", ylab="first fixation duration [ms]",  main='IOVP (SFD), wl>=3 & wl<=12')

# McConkie-type plots
	library(MASS)

	launchsite <- (a$let - a$ao1)

	ix5 <- ((1/a$l - 1) == 5)  & a$Ffix
	ix7 <- ((1/a$l - 1) == 7)  & a$Ffix
	ix9 <- ((1/a$l - 1) == 9)  & a$Ffix
	lengthixs <- list(ix5, ix7, ix9)

	launchsites <- c(-1, -2, -3, -5)
	ixls1 <- launchsite>-1.5 & launchsite <= 0.5
	ixls2 <- launchsite>-2.5 & launchsite<=-1.5  
	ixls3 <- launchsite>-3.5 & launchsite<=-2.5 
	ixls5 <- launchsite>-5.5 & launchsite<=-4.5
	lsixs <- list(ixls1, ixls2, ixls3, ixls5)

	ttls <- c(5,7,9)
	Ncuts <- 10
	quartz()
	par(mfrow = c(4,3))
	for ( j in 1:4 ) {
		ixls <- lsixs[[j]]
		for (i in 1:3) {
			ixl <- lengthixs[[i]] 
			ix <- ixls & ixl & a$dir==1
			ix <- ix & !is.na(ix)

			q <- cut(a$o[ix], Ncuts, labels = FALSE)
			pvp <- aggregate(a$o[ix], list(q), length)
			if  (j==1) {ttl <- ttls[i]} else {ttl <- NULL}
			if  (i==1) {ylb <- as.character(launchsites[j])}
			else {ylb <- "frequency"	}
			xlb <- "ILP"
			plot(pvp$x, type="p", main = ttl, xlab=xlb, ylab=ylb, ylim=c(0,50))

			xx <- fitdistr(a$o[ix], "normal", na.rm=TRUE)
			rx <- diff(range(q))
			curve(sum(ix)*dnorm(x, mean=xx$estimate[["mean"]] * rx + min(q), sd=xx$estimate[["sd"]]*rx) , from=min(q), to=max(q), add=TRUE)
		}
	}

}


