\name{eyetrackR-package}
\alias{eyetrackR-package}
\alias{eyetrackR}
\docType{package}
\encoding{UTF-8}
\title{
Analysis of eye movement data
}
\description{
eyetrackR provides functions for the analysis of eyetracker data. Currently it is geared towards, but not limited to, eye movements during reading. Starting from the raw time series of x and y samples, the package allows automated classification into saccades and fixations, with adjustable saccade detection parameters. Saccade detection results in a fixation sequence. From this, given additional input about the reading material, aggregate measures such as gaze durations (dwell time on a word) or skipping probabilities are computed. The data are shaped into a format that facilitates analyses using a distributed-processing approach, i.e., taking into account influences of the neighbouring words (or fixations) on the current fixation duration. Sample analyses demonstrate how these measures can be analyzed as a function of word and reader parameters, using linear mixed effects models.

See help files for individual parts listed below.
}
\details{
\tabular{ll}{
Package: \tab eyetrackR\cr
Type: \tab Package\cr
Version: \tab 0.13\cr
Date: \tab 2009-02-08\cr
License: \tab GPL (>= 2) \cr
LazyLoad: \tab yes\cr
}

\enumerate{
\item \emph{(data preprocessing)}\cr
	\itemize{	
	\item (edf to ascii conversion, .dat, .msg, and .msg2 file creation and cleanup)\cr
		preprocess\cr
	\item (case management)\cr
		xallsac_l3 (demo)\cr 
		\itemize{	
			\item (message file parser)\cr
			parse_msg_file (should be a demo)\cr
				parms1 \code{ # usage: data(parms1); contains detection parameters }\cr
				readSubjects\cr
			\item ((micro-)saccade detection)\cr
				vecvel\cr
				microsacc\cr
				binsacc\cr
				saccpar\cr
		}
	}
\item (analysis of fixation sequences)\cr
	currently only contains demos
	\itemize{
		\item xSetup1_l3 (demo)\cr
		\item xIndex_fix_l2_logical (demo)
		\item xSetup2_l3 (demo)\cr
		% xSetup2\cr
		% xSetup3\cr
	}
\item (definition of aggregate measures)\cr
	currently only contains demos
	\itemize{
		\item xSetup2.2_dvmat_l3.R (demo)\cr
	}
	% xProbDur\cr
\item (sample statistical analysis)\cr
}
}

\author{
Jochen Laubrock with Reinhold Kliegl, Ralf Engbert, Sarah Risse, Daniel Schad, Michael Dambacher

Maintainer: Jochen Laubrock <laubrock@uni-potsdam.de>
}
\references{
Laubrock & Kliegl (in preparation). eyetrackR: an R package for the analysis of eye movement data.

Engbert, R., & Mergenthaler, K. (2006) Microsaccades are triggered by low retinal image slip. \emph{Proceedings of the National Academy of Sciences of the United States of America}, \bold{103}, 7192--7197.

%Engbert, R. & Kliegl, R. (2003) Microsaccades uncover the orientation of covert attention. \emph{Vision Research}, \bold{43}, 1035--1045. 

}

\keyword{ package }
\seealso{\code{\link{vecvel}}, \code{\link{microsacc}}, \code{\link{binsacc}}, \code{\link{saccpar}}}


\examples{
 #############################################################
 ## (micro-)saccade detection
 #############################################################
data(trial4)
 # names(trial4) <- c("t", "xl", "yl", "pl", "xr", "yr", "pr")

SAMPLING <- 500
VFAC <- 5
MINDUR <- 3

vl <- vecvel(trial4[,c("xl", "yl")], SAMPLING, 2)
sacl <- microsacc(trial4[,c("xl", "yl")], vl, VFAC, MINDUR)

vr <- vecvel(trial4[,c("xr", "yr")], SAMPLING, 2)
sacr <- microsacc(trial4[,c("xr", "yr")], vr, VFAC, MINDUR)

sacb <- binsacc(sacl$sac, sacr$sac)
sacb2 <- saccpar(sacb$sac)

 #############################################################
 ## sentence corpus
 #############################################################
data(PSC2.DWDS)
example(PSC2.DWDS)

}
