\name{readSubjects}
\alias{readSubjects}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ read subject list }
\description{ Utility function that reads a list of subject codes from a file and checks for availability of the corresponding data files. The user must generate a control file containing a list of subjects participating in the experiment, one subject code per line. It is assumed that the file containing the subject list is labeled [exp_str].subjects, and the corresponding data files are labeled [subject-code].dat and [subject-code].msg for raw data (time series) and message files, respectively.

readSubjects  assumes that files have endings '.dat' for raw data and '.msg2' for (filtered) messages as provided by preprocess.R

For example, if an experiment is called 'P2', and the subject codes are 'PSC2x01', 'PSC2x02', ..., 'PSC2x6', then the subject list must reside in a file called PSC2.subjects, and look like the following

PSC2x1\cr
PSC2x2\cr
PSC2x3\cr
PSC2x4\cr
PSC2x5\cr
PSC2x6\cr
}
\usage{
readSubjects(exp_str, dir='./data', msgdir = dir, datdir = dir)
}
\arguments{
  \item{exp_str}{ prefix of subject file (name before '.subjects' suffix) }
  \item{dir}{ path to directory with subjects list }
  \item{msgdir}{ path to directory with msg files, defaults to dir}
  \item{datdir}{ path to directory with dat files, defaults to dir }
}
% \details{
%   ~~ If necessary, more details than the description above ~~
% }
\value{
	subject: a vector containing the subject codes
}
% \references{ ~put references to the literature/web site here ~ }
\author{ Jochen Laubrock }
\note{ Often you might not need the additional checking, and can use builtin
subjects <- readLines(con=full_path_to_subject_file) instead
}
\seealso{ \code{\link{preprocess}}}
\examples{
# create a dummy subject list and directory infrastructure
estr <- 'P2'
baseStr <- 'PSC2x'; 
basedir <- '/tmp'
tmpfile = file.path(basedir, paste(estr,'.subjects', sep=''))
outf <- file(tmpfile, open='w')
for (i in 1:6){ 
	tmpstr <- sprintf("\%s\%3d", baseStr, i)
	writeLines(tmpstr,  outf) 
}
close(outf)

# read subject list
\donttest{
subjects = readSubjects(estr, dir='/tmp')
print(subjects)
}

# delete dummy files
unlink(tmpfile)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
