\name{wordLetterPos}
\alias{wordLetterPos}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Convert fixation position [pix] to word and letter position }
\description{
Fractional letters are used to avoid (accumulation of) rounding errors
}
\usage{
wordLetterPos(fpos, sWL, PIX_PER_CHAR, wbr_all=NULL, wbl_all=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fpos}{fixation position (relative to sentence onset)}
  \item{sWL}{vector containing word length information (in letters) for the current sentence (including punctuating, but excluding spaces)}
  \item{PIX_PER_CHAR}{ number of pixels per character (how many pixel make up a character) }
  \item{wbr_all}{ optional vector containing left word boundaries for all words  }
  \item{wbl_all}{ optional vector containing right word boundaries for all words }

}
\details{
  Note that fixation position is expected relative to sentence onset--thus offset must be subtracted from absolute fixation position in calling wordLetterPos.

Note that a fixed-width font is required if sWL and PIX_PER_CHAR arguments are to be used. Alternatively, wbl_all and wbr_all can be provided, in which case sWL and PIX_PER_CHAR are not used. The latter procedure can be used with proportional fonts, for which vectors of word boundaries must be externally provided (e.g., extracted from Data Viewer IAS files)

Fixation position to the left of the sentence are assigned NA.

Fixation position to the right of the sentence are assigned NA.
}
\value{
  \item{w}{Fixated word}
  \item{l}{Fixated letter in word}
}
% \references{ ~put references to the literature/web site here ~ }
\author{ Jochen Laubrock }
\note{ complete rewrite of Antje Nuthmann's Matlab version.

Required for computation of preferred viewing location, optimal viewing position effects, etc.
}
\seealso{ \code{\link{trial32_sn140}}, \code{\link{sac_trial32_sn140}}}
\examples{
	data(trial32_sn140)
	data(sac_trial32_sn140)
	data(parms1)
	data(PSC2.DWDS)

	sac <- sac_trial32_sn140
	xpix <- trial32_sn140$xl
	sentenceInCorpusIx <- PSC2.DWDS$sn==140
	sentenceWL <- PSC2.DWDS[sentenceInCorpusIx, "nlett1"]

	wl <- matrix(nrow=nrow(sac),ncol=2)

	# initial fixation
	fpos0 <- mean( xpix[ 1:sac[1,1] ] )
	res <- wordLetterPos(fpos0 - LEFT_SENTENCE_OFFSET, sentenceWL, PIX_PER_CHAR)
	wl[1,1] <- res$w
	wl[1,2] <- res$l

	# fixations between saccades
	for (i in 1:(nrow(sac)-1)) {
		fpos <- mean( xpix[ sac[i,2]:sac[i+1,1] ] )
		res <- wordLetterPos(fpos - LEFT_SENTENCE_OFFSET, sentenceWL, PIX_PER_CHAR)
		wl[i+1,1] <- res$w
		wl[i+1,2] <- res$l
	}
	
	# ignore fixation after final saccade
	wl <- data.frame(wl)
	names(wl) <- c("wrd", "ltr")
	print(wl)
	
	
	# check whether fixations just before the first letter of a word 
	# are categorized as belonging to the space before the word
	wpll<-NULL
	initialLetters <- cumsum(c(0, sentenceWL + 1)) - .01 
	for (fpos in initialLetters * PIX_PER_CHAR) {
		wpll <- rbind(wpll, wordLetterPos(fpos, sentenceWL, PIX_PER_CHAR))
	}
	nextWL <- c(sentenceWL, NA)
	cbind(wpll, nextWL, initialLetters)
	
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
